from __future__ import absolute_import
from nlpaug.augmenter.word.word_augmenter import *
from nlpaug.augmenter.word.random import *
from nlpaug.augmenter.word.word_embs import *
from nlpaug.augmenter.word.tfidf import *
from nlpaug.augmenter.word.spelling import *
from nlpaug.augmenter.word.context_word_embs import *
from nlpaug.augmenter.word.synonym import *
from nlpaug.augmenter.word.antonym import *
from nlpaug.augmenter.word.split import *
